Package: lares
Type: Package
Title: Lean Analytics and Robust Exploration Sidekick
Version: 5.4.0
Authors@R: c(
    person("Bernardo", "Lares", , "laresbernardo@gmail.com", c("aut", "cre")))
Maintainer: Bernardo Lares <laresbernardo@gmail.com>
Description: Auxiliary package for better/faster analytics, visualization, data mining, and machine
    learning tasks. With a wide variety of family functions, like Machine Learning, Data Wrangling,
    Marketing Mix Modeling (Robyn), Exploratory, API, and Scrapper, it helps the analyst or 
    data scientist to get quick and robust results, without the need of repetitive coding or 
    advanced R programming skills.
Depends: R (>= 3.5.0)
Imports: dplyr (>= 1.0.0), ggplot2 (>= 3.4.0), httr (>= 1.4.3),
        jsonlite (>= 1.8.0), lubridate (>= 1.7.9), openxlsx (>=
        3.5.10), patchwork (>= 1.3.0), pROC (>= 1.18.5), rlang (>=
        0.6.3), rpart (>= 4.1.0), rpart.plot (>= 3.1.0), rvest (>=
        1.0.0), stringr (>= 1.4.2), tidyr (>= 1.0.0), yaml (>= 2.2.4)
Suggests: beepr (>= 0.1.5), DALEX (>= 1.0.0), fansi (>= 1.0.0),
        googleAuthR (>= 1.0.0), googlesheets4 (>= 1.0.0), h2o (>=
        3.24.0), knitr (>= 1.31.0), MASS, quantmod (>= 0.4.20),
        rmarkdown (>= 2.7)
VignetteBuilder: knitr
URL: https://github.com/laresbernardo/lares,
        https://laresbernardo.github.io/lares/
BugReports: https://github.com/laresbernardo/lares/issues
RoxygenNote: 7.3.3
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-23 06:54:57 UTC; bernardo
Author: Bernardo Lares [aut, cre]
Repository: CRAN
Date/Publication: 2026-04-23 07:40:02 UTC
Built: R 4.5.2; ; 2026-04-24 13:30:03 UTC; unix
