# Generated by roxygen2: do not edit by hand

S3method(plot,corr_var)
S3method(plot,h2o_automl)
S3method(plot,h2o_shap)
S3method(plot,robyn_modelselector)
S3method(plot,stocks_hist)
S3method(plot,tree_var)
S3method(plot,x2y)
S3method(plot,x2y_preds)
S3method(print,h2o_automl)
S3method(print,maze_solve)
S3method(print,robyn_crossmmm)
S3method(print,wordle_check)
S3method(print,wordle_simulation)
export("%>%")
export(ROC)
export(are_binary)
export(are_constant)
export(are_id)
export(autoline)
export(balance_data)
export(bind_files)
export(bring_api)
export(cache_clear)
export(cache_exists)
export(cache_pipe)
export(cache_read)
export(cache_write)
export(cal_split)
export(categ_reducer)
export(check_attr)
export(check_opts)
export(chr2date)
export(chr2logical)
export(chr2num)
export(ci_lower)
export(ci_upper)
export(ci_var)
export(cleanNames)
export(cleanText)
export(clusterKmeans)
export(clusterOptimalK)
export(clusterVisualK)
export(conf_mat)
export(corr)
export(corr_cross)
export(corr_var)
export(cran_logs)
export(crosstab)
export(daily_portfolio)
export(daily_stocks)
export(dalex_local)
export(dalex_residuals)
export(dalex_variable)
export(date_cuts)
export(date_feats)
export(db_download)
export(db_upload)
export(df_str)
export(dist2d)
export(distr)
export(dont_sleep)
export(dont_sleep_time)
export(encrypt_file)
export(errors)
export(etf_sector)
export(export_plot)
export(export_results)
export(fb_accounts)
export(fb_ads)
export(fb_creatives)
export(fb_insights)
export(fb_process)
export(fb_report_check)
export(fb_rf)
export(fb_token)
export(file_name)
export(file_type)
export(filesGD)
export(files_functions)
export(font_exists)
export(forecast_arima)
export(formatColoured)
export(formatHTML)
export(formatNum)
export(freqs)
export(freqs_df)
export(freqs_list)
export(freqs_plot)
export(gain_lift)
export(gemini_ask)
export(gemini_image)
export(get_credentials)
export(get_creds)
export(get_currency)
export(get_tweets)
export(gg_colour_customs)
export(gg_fill_customs)
export(gg_text_customs)
export(gg_vals)
export(glued)
export(gpt_ask)
export(gpt_classify)
export(gpt_convert)
export(gpt_extract)
export(gpt_format)
export(gpt_history)
export(gpt_prompter)
export(gpt_table)
export(gpt_tag)
export(gpt_translate)
export(grepl_letters)
export(grepm)
export(gtrends_related)
export(gtrends_time)
export(h2o_automl)
export(h2o_explainer)
export(h2o_predict_API)
export(h2o_predict_MOJO)
export(h2o_predict_binary)
export(h2o_predict_model)
export(h2o_results)
export(h2o_selectmodel)
export(h2o_shap)
export(haveInternet)
export(hex_to_raw)
export(holidays)
export(image_metadata)
export(importxlsx)
export(impute)
export(install_recommended)
export(ip_data)
export(is_even)
export(is_ip)
export(is_odd)
export(is_url)
export(iter_seeds)
export(json2vector)
export(lares_logo)
export(lares_pal)
export(lasso_vars)
export(left)
export(list_cats)
export(listfiles)
export(loglossBinary)
export(mae)
export(mail_send)
export(mape)
export(markdown2df)
export(maze_gridsearch)
export(maze_solve)
export(missingness)
export(model_metrics)
export(model_preprocess)
export(move_files)
export(mp3_get)
export(mp3_trim)
export(mp3_update_tags)
export(mplot_conf)
export(mplot_cuts)
export(mplot_cuts_error)
export(mplot_density)
export(mplot_full)
export(mplot_gain)
export(mplot_importance)
export(mplot_lineal)
export(mplot_metrics)
export(mplot_response)
export(mplot_roc)
export(mplot_splits)
export(mplot_topcats)
export(mse)
export(msplit)
export(myip)
export(ngrams)
export(noPlot)
export(normalize)
export(num_abbr)
export(numericalonly)
export(ohe_commas)
export(ohse)
export(outlier_tukey)
export(outlier_turkey)
export(outlier_zscore)
export(outlier_zscore_plot)
export(plot_cats)
export(plot_chord)
export(plot_df)
export(plot_nums)
export(plot_palette)
export(plot_survey)
export(plot_timeline)
export(prophesize)
export(quants)
export(queryDB)
export(queryGA)
export(quiet)
export(raw_to_hex)
export(read.file)
export(readGS)
export(read_encrypted)
export(reduce_pca)
export(reduce_tsne)
export(remove_stopwords)
export(removenacols)
export(removenarows)
export(replaceall)
export(replacefactor)
export(right)
export(rmse)
export(robyn_hypsbuilder)
export(robyn_marginal)
export(robyn_modelselector)
export(robyn_performance)
export(robyn_xchannels)
export(robyn_xmodels)
export(rsq)
export(rsqa)
export(rtistry_sphere)
export(scale_x_abbr)
export(scale_x_comma)
export(scale_x_dollar)
export(scale_x_formatNum)
export(scale_x_percent)
export(scale_y_abbr)
export(scale_y_comma)
export(scale_y_dollar)
export(scale_y_formatNum)
export(scale_y_percent)
export(scrabble_dictionary)
export(scrabble_points)
export(scrabble_score)
export(scrabble_words)
export(sentimentBreakdown)
export(seq_surnames)
export(shap_var)
export(slackSend)
export(splot_change)
export(splot_divs)
export(splot_etf)
export(splot_growth)
export(splot_roi)
export(splot_summary)
export(splot_types)
export(spread_list)
export(statusbar)
export(stocks_file)
export(stocks_hist)
export(stocks_obj)
export(stocks_quote)
export(stocks_report)
export(sudoku_solver)
export(target_set)
export(textCloud)
export(textFeats)
export(textTokenizer)
export(theme_lares)
export(tic)
export(toc)
export(toon_reduction)
export(topics_rake)
export(tree_var)
export(try_require)
export(updateLares)
export(v2t)
export(vector2text)
export(warnifnot)
export(weighted_value)
export(what_size)
export(winsorize)
export(wordle_check)
export(wordle_dictionary)
export(wordle_simulation)
export(writeGS)
export(write_encrypted)
export(x2y)
export(x2y_metric)
export(x2y_preds)
export(year_month)
export(year_quarter)
export(year_week)
export(zerovar)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_match)
importFrom(dplyr,case_when)
importFrom(dplyr,contains)
importFrom(dplyr,count)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,distinct_at)
importFrom(dplyr,ensym)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,lag)
importFrom(dplyr,last)
importFrom(dplyr,left_join)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,mutate_at)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n)
importFrom(dplyr,ntile)
importFrom(dplyr,one_of)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,rename_at)
importFrom(dplyr,right_join)
importFrom(dplyr,row_number)
importFrom(dplyr,rowwise)
importFrom(dplyr,sample_n)
importFrom(dplyr,select)
importFrom(dplyr,select_if)
importFrom(dplyr,slice)
importFrom(dplyr,slice_head)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarise)
importFrom(dplyr,summarise_all)
importFrom(dplyr,summarise_if)
importFrom(dplyr,summarize)
importFrom(dplyr,summarize_all)
importFrom(dplyr,sym)
importFrom(dplyr,tally)
importFrom(dplyr,tibble)
importFrom(dplyr,transmute_if)
importFrom(dplyr,ungroup)
importFrom(dplyr,vars)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,enquo)
importFrom(ggplot2,enquos)
importFrom(ggplot2,expand_limits)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_raster)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guide_colourbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,last_plot)
importFrom(ggplot2,margin)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,position_fill)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,position_stack)
importFrom(ggplot2,quo_name)
importFrom(ggplot2,quos)
importFrom(ggplot2,rel)
importFrom(ggplot2,scale_color_brewer)
importFrom(ggplot2,scale_color_identity)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_brewer)
importFrom(ggplot2,scale_colour_continuous)
importFrom(ggplot2,scale_colour_gradient)
importFrom(ggplot2,scale_colour_identity)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_brewer)
importFrom(ggplot2,scale_fill_continuous)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_grey)
importFrom(ggplot2,scale_fill_identity)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_shape_identity)
importFrom(ggplot2,scale_size)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_date)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_x_reverse)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,sec_axis)
importFrom(ggplot2,stat_density_2d)
importFrom(ggplot2,stat_ecdf)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,unit)
importFrom(ggplot2,update_geom_defaults)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(grDevices,dev.off)
importFrom(grDevices,dev.size)
importFrom(grDevices,graphics.off)
importFrom(grDevices,png)
importFrom(graphics,box)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,mtext)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,rasterImage)
importFrom(graphics,rect)
importFrom(httr,GET)
importFrom(httr,POST)
importFrom(httr,add_headers)
importFrom(httr,authenticate)
importFrom(httr,config)
importFrom(httr,content)
importFrom(httr,http_error)
importFrom(httr,oauth1.0_token)
importFrom(httr,oauth_app)
importFrom(httr,oauth_endpoint)
importFrom(httr,set_config)
importFrom(httr,stop_for_status)
importFrom(httr,upload_file)
importFrom(jsonlite,flatten)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(lubridate,"%m+%")
importFrom(lubridate,"%m-%")
importFrom(lubridate,ceiling_date)
importFrom(lubridate,date)
importFrom(lubridate,day)
importFrom(lubridate,days)
importFrom(lubridate,dmy)
importFrom(lubridate,dmy_hms)
importFrom(lubridate,floor_date)
importFrom(lubridate,hour)
importFrom(lubridate,minute)
importFrom(lubridate,month)
importFrom(lubridate,second)
importFrom(lubridate,wday)
importFrom(lubridate,week)
importFrom(lubridate,weeks)
importFrom(lubridate,year)
importFrom(lubridate,years)
importFrom(lubridate,ymd)
importFrom(lubridate,ymd_hms)
importFrom(openxlsx,addWorksheet)
importFrom(openxlsx,copyWorkbook)
importFrom(openxlsx,getSheetNames)
importFrom(openxlsx,loadWorkbook)
importFrom(openxlsx,read.xlsx)
importFrom(openxlsx,removeWorksheet)
importFrom(openxlsx,renameWorksheet)
importFrom(openxlsx,saveWorkbook)
importFrom(openxlsx,sheets)
importFrom(openxlsx,write.xlsx)
importFrom(pROC,ci)
importFrom(pROC,roc)
importFrom(patchwork,guide_area)
importFrom(patchwork,plot_annotation)
importFrom(patchwork,plot_layout)
importFrom(patchwork,wrap_plots)
importFrom(rlang,.data)
importFrom(rlang,as_label)
importFrom(rpart,rpart)
importFrom(rpart,rpart.control)
importFrom(rpart.plot,rpart.plot)
importFrom(rpart.plot,rpart.rules)
importFrom(rvest,html_attr)
importFrom(rvest,html_attrs)
importFrom(rvest,html_node)
importFrom(rvest,html_nodes)
importFrom(rvest,html_table)
importFrom(rvest,html_text)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,dist)
importFrom(stats,end)
importFrom(stats,kmeans)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,start)
importFrom(stats,var)
importFrom(stats,xtabs)
importFrom(stringr,fixed)
importFrom(stringr,str_count)
importFrom(stringr,str_length)
importFrom(stringr,str_pad)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_to_title)
importFrom(stringr,word)
importFrom(tidyr,gather)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,spread)
importFrom(tidyr,unite)
importFrom(utils,URLencode)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,download.file)
importFrom(utils,find)
importFrom(utils,flush.console)
importFrom(utils,getParseData)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,modifyList)
importFrom(utils,object.size)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,remove.packages)
importFrom(utils,tail)
importFrom(utils,type.convert)
importFrom(utils,write.csv)
importFrom(utils,write.table)
importFrom(yaml,read_yaml)
