% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/other_functions.R
\name{lares-exports}
\alias{lares-exports}
\alias{\%>\%}
\title{Pipe operator}
\description{
Pipe operator
}
